#perl feature.pl -o hexoses.data input/glucose_t* input/galactose_t* input/mannose_t* input/6carbs_t* input/binding_t* input/random_t* > hexoses.files


use lib "../perl_lib";
use strict;
use warnings;
use Carp;
use Structure_Mod::IO_Mod;
use Binding_Site::Sphere;
use Feature_Vector::Feature_Vector;
use Feature_Vector::ILP_Repr;

my $PDBDIR = 'samples';
#my $PDBDIR = '/mnt/hdb5/pdb/entries';
my $OUTPUT = "STDOUT";

$| = 1;
eval (&main());


=head2 main()

 Title   : main
 Usage   : &main();
 Function: Reads input files, process the PDB entries, and write 
           output on output files.
 Returns : Nothing. Writes output.
 Args    : -o output_file: defaults to STDOUT.
           a list of input files containing PDB entries and their
                                 center coordinates.

=cut

sub main(){
  my @args = @ARGV; #store the args array
  my $number = 1; #the current PDB entry number

  #for each file
  my $out = 0;
  foreach my $file (@args){
    #check if output file
    if ($file eq '-o'){
      $out = 1;
      next;
    }
    if ($out){
      $OUTPUT = $file;
      $out = 0;
      next;
    }

    # load input file
    open (INPUT, "$file") || confess "couldn't open $file: $!. Stopped";

    while (my $line = <INPUT>){
      chomp $line;
      next if $line =~ /^#/; #rule out comments
      next if $line =~ /^\W*$/; #rule out blank lines
      # split the input line into its constituents
      my ($file, $label, $method, $include, $discard, @elems) = 
	split /\s+/, $line;

      #my $foldername = substr ($line, 1, 2);
      substr ($file, 4, 5, '');
      my $gfile = "pdb$file.gz";
      #$file = "$PDBDIR/$foldername/$gfile";
      my $dfile = "$PDBDIR/$gfile";

      print "$number $file\n";
      system ("cp -f $dfile $file.gz");
      system ("gunzip -f $file.gz");

      #get the protein structure
      my $in  = Structure_Mod::IO_Mod->new('-file' => $file,
					   '-format' => 'pdb');
      $in->_noheader(1);#to skip storing header records
      my $struc = $in->next_structure(); #get the entry object
      system ("rm -f $file");

      # check for models
      my @models = $struc->get_models($struc);
      warn "ATT: more than one model (", @models ," models)\n" if $#models > 0;

      my $sphere = Binding_Site::Sphere->new('-structure' => $struc,
					       '-radius' => 10,
                                               '-width' => 1,
                                               '-first_layer_width' => 3);
      $sphere->compute_all($method, [splice @elems, 0, $include], 
			   [splice @elems, 0, $discard]);
      #print join (' ', $sphere->get_center()), "\n";

=for comment
      my $vector = Feature_Vector::Feature_Vector->new('-sphere' => $sphere,
						       '-binary' => 0);
      $vector->compute_atomic("charge");
      $vector->compute_atomic("hbond");
      $vector->compute_atomic("hydrophob");
      $vector->compute_residue("detailed");
      #$vector->compute_atom_nb();
      #$vector->normalize;
=cut
      
#######################################################################
### ILP
#######################################################################
      
      #ATT: make sure to delete bind.yap at each run

      #higly specific to the 160hexoses data
      my $nbToPass = $number;
      $nbToPass = $number - 80 if $number > 80;

      my $ilp = Feature_Vector::ILP_Repr->
	  new('-sphere' => $sphere,
	      '-property'=> [("charge","hbond","hydrophob")],
	      '-output' => "bind.yap",
	      '-label'=> $label,
	      '-number'=>$nbToPass,
	      );
      $ilp->compute_atomic;

######################################################################


=for comment
      #create output file and write the header
      if ($number == 1){
	no strict 'refs'; # so that symbolic references can be used
	# symbolic references
	my $fh = "STDOUT"; # used as a symbolic reference file handle
	if ($OUTPUT ne "STDOUT"){
	  open (FH, "> $OUTPUT") || 
	    confess "couldn't open $OUTPUT: $!. Stopped";
	  $fh = "FH";
	}
	print $fh "LABEL,", join (',', $vector->get_feature_titles()), "\n";
	close FH;
      }

      $vector->print_feature_vector("$label", "$OUTPUT", ',');
=cut
      $number++;
    }
    close INPUT;
  }
}#sub main()


